@@ -11,6 +11,7 @@ from importlib import import_module
 from django.conf import settings
 from django.utils.deprecation import MiddlewareMixin
 from django.contrib.sessions.middleware import SessionMiddleware as BaseSessionMiddleware
+from django.core.exceptions import ImproperlyConfigured
 
 
 def _parse_key(key):
@@ -19,7 +20,7 @@ def _parse_key(key):
             k = open(k, 'rb').read()
         return k
 
-    if type(key) is tuple:
+    if isinstance(key, tuple):
         # Key pair.
         return _load_key(key[0]), _load_key(key[1]), 'RS256'
 
@@ -27,12 +28,65 @@ def _parse_key(key):
     return key, key, 'HS256'
 
 
+def _parse_fields(fields):
+    "Parse and validate field definitions."
+    snames, lnames = [], []
+
+    for i, field in enumerate(fields):
+        # Transform field in 3-tuple.
+        if isinstance(field, tuple):
+            if len(field) == 2:  # (attrname, sname)
+                field = (field[0], field[1], field[1])
+
+            elif len(field) == 1:  # (attrname)
+                field = (field[0], field[0], field[0])
+
+        else:  # attrname
+            field = (field, field, field)
+
+        # Collect all snames and lnames for uniqueness check.
+        snames.append(field[1])
+        lnames.append(field[2])
+
+        # Validate that "sk" is not used, we use that for the session key.
+        if field[1] == SESSION_FIELD:
+            raise ImproperlyConfigured(
+                'Short name "%s" is reserved for session field. Use '
+                'DJANGO_SESSION_JWT["SESSION_FIELD"] to specify another '
+                'value.' % SESSION_FIELD)
+
+        if len(field) != 3:
+            raise ImproperlyConfigured(
+                'DJANGO_SESSION_JWT["FIELDS"] should be a list of 3-tuples.')
+
+        fields[i] = field
+
+    if len(snames) != len(set(snames)):
+        raise ImproperlyConfigured(
+            'DJANGO_SESSION_JWT["FIELDS"] short names are not unique')
+
+    if len(lnames) != len(set(lnames)):
+        raise ImproperlyConfigured(
+            'DJANGO_SESSION_JWT["FIELDS"] long names are not unique')
+
+    return fields
+
+
+SESSION_FIELD = getattr(settings, 'DJANGO_SESSION_JWT', {}).get('SESSION_FIELD', 'sk')
 KEY, PUBKEY, ALGO = _parse_key(getattr(settings, 'DJANGO_SESSION_JWT', {}).get('KEY', settings.SECRET_KEY))
-FIELDS = getattr(settings, 'DJANGO_SESSION_JWT', {}).get('FIELDS', [])
+FIELDS = _parse_fields(getattr(settings, 'DJANGO_SESSION_JWT', {}).get('FIELDS', []))
 LOGGER = logging.getLogger(__name__)
 LOGGER.addHandler(logging.NullHandler())
 
 
+def rgetattr(obj, name):
+    "Recursive getattr()."
+    names = name.split('.')
+    for n in names:
+        obj = getattr(obj, n)
+    return obj
+
+
 def verify_jwt(blob):
     """
     Verify a JWT and return the session_key attribute from it.
@@ -44,9 +98,10 @@ def verify_jwt(blob):
         return {}
 
     # Convert short names to long names.
-    for lname, sname in [(i[0], i[1]) for i in FIELDS if type(i) is tuple]:
+    for _, sname, lname in FIELDS:
         try:
-            fields[lname] = fields.pop(sname)
+            # Leave both short and long forms in dictionary.
+            fields[lname] = fields[sname]
 
         except KeyError:
             continue
@@ -59,7 +114,7 @@ def create_jwt(user, session_key, expires=None):
     Create a JWT for the given user containing the configured fields.
     """
     fields = {
-        'sk': session_key,
+        SESSION_FIELD: session_key,
         'iat': datetime.utcnow(),
     }
     if expires:
@@ -67,19 +122,14 @@ def create_jwt(user, session_key, expires=None):
         # Django 2.0, 1.11 have - chars in date...
         expires = expires.replace('-', ' ')
         fields['exp'] = datetime.strptime(expires, '%a, %d %b %Y %H:%M:%S %Z')
-    for field_name in FIELDS:
-        if type(field_name) is tuple:
-            lname, sname = field_name
-        
-        else:
-            lname = sname = field_name
 
+    for attrname, sname, _ in FIELDS:
         try:
-            fields[sname] = getattr(user, lname)
+            fields[sname] = rgetattr(user, attrname)
 
         except AttributeError:
             # Omit missing fields:
-            LOGGER.warning('Could not get missing field %s from user', field_name)
+            LOGGER.warning('Could not get missing field %s from user', attrname)
             continue
 
     return jwt.encode(fields, KEY, algorithm=ALGO).decode('utf8')
@@ -98,14 +148,22 @@ class SessionMiddleware(BaseSessionMiddleware):
 
     def process_request(self, request):
         session_jwt = request.COOKIES.get(settings.SESSION_COOKIE_NAME)
-        session_key = verify_jwt(session_jwt).get('sk')
+        fields = verify_jwt(session_jwt)
+        session_key = fields.pop(SESSION_FIELD, None)
         request.session = self.SessionStore(session_key)
+        request.session['jwt'] = fields
 
     def process_response(self, request, response):
         # Rather than duplicating the session logic here, just allow super()
         # to do it's thing, then convert the session cookie (if any) when it's
         # done.
         super(SessionMiddleware, self).process_response(request, response)
+
+        # Behave the same as contrib.sessions, only recreate the JWT if the session
+        # was modified or SESSION_SAVE_EVERY_REQUEST is enabled.
+        if not request.session.modified and not settings.SESSION_SAVE_EVERY_REQUEST:
+            return response
+
         try:
             convert_cookie(response.cookies, request.user)
 
